@@ -2,19 +2,19 @@
 #
 # Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
 #
-# This file is part of Launch Control.
+# This file is part of Lava Dashboard.
 #
-# Launch Control is free software: you can redistribute it and/or modify
+# Lava Dashboard is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# Launch Control is distributed in the hope that it will be useful,
+# Lava Dashboard is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.
+# along with Lava Dashboard.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Unit tests of the Dashboard application
@@ -486,8 +486,7 @@ class BundleDeserializerSuccessTests(TestCaseWithScenarios):
 
     def test_TestResult__measurement(self):
         self.assertEqual(
-            self.s_test_result.measurement,
-            decimal.Decimal("1000.3"))
+            self.s_test_result.measurement, "1000.3")
 
     def test_TestResult__units(self):
         self.assertEqual(self.s_test_result.units, "bogomips")
